@@ -165,14 +165,13 @@ theme_thumbnail_generate (GnomeThemeMetaInfo *info, AppearanceData *data)
 }
 
 static void
-theme_changed_on_disk_cb (GnomeThemeType       type,
-			  gpointer             theme,
-			  GnomeThemeChangeType change_type,
-			  GnomeThemeElement    element,
+theme_changed_on_disk_cb (GnomeThemeCommonInfo *theme,
+			  GnomeThemeChangeType  change_type,
+			  GnomeThemeElement     element,
 			  AppearanceData       *data)
 {
-  if (type == GNOME_THEME_TYPE_METATHEME) {
-    GnomeThemeMetaInfo *meta = theme;
+  if (theme->type == GNOME_THEME_TYPE_METATHEME) {
+    GnomeThemeMetaInfo *meta = (GnomeThemeMetaInfo *) theme;
 
     if (change_type == GNOME_THEME_CHANGE_CREATED) {
       gtk_list_store_insert_with_values (data->theme_store, NULL, 0,
@@ -211,23 +210,6 @@ get_default_string_from_key (GConfClient *client, const char *key)
   return str;
 }
 
-static gint
-get_default_int_from_key (GConfClient *client, const char *key, gint def)
-{
-  GConfValue *value;
-  gint val = def;
-
-  value = gconf_client_get_default_from_schema (client, key, NULL);
-
-  if (value) {
-    if (value->type == GCONF_VALUE_INT)
-      val = gconf_value_get_int (value);
-    gconf_value_free (value);
-  }
-
-  return val;
-}
-
 /* Find out if the lockdown key has been set. Currently returns false on error... */
 static gboolean
 is_locked_down (GConfClient *client)
@@ -235,39 +217,43 @@ is_locked_down (GConfClient *client)
   return gconf_client_get_bool (client, LOCKDOWN_KEY, NULL);
 }
 
-static void
-theme_load_from_gconf (GConfClient *client, GnomeThemeMetaInfo *theme)
+static GnomeThemeMetaInfo *
+theme_load_from_gconf (GConfClient *client)
 {
+  GnomeThemeMetaInfo *theme;
   gchar *scheme;
 
-  g_free (theme->gtk_theme_name);
+  theme = gnome_theme_meta_info_new ();
+
   theme->gtk_theme_name = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
+  if (theme->gtk_theme_name == NULL)
+    theme->gtk_theme_name = g_strdup ("Clearlooks");
 
-  g_free (theme->gtk_color_scheme);
   scheme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
-
   if (scheme == NULL || !strcmp (scheme, "")) {
     g_free (scheme);
     scheme = gtkrc_get_color_scheme_for_theme (theme->gtk_theme_name);
   }
   theme->gtk_color_scheme = scheme;
 
-  g_free (theme->metacity_theme_name);
   theme->metacity_theme_name = gconf_client_get_string (client, METACITY_THEME_KEY, NULL);
+  if (theme->metacity_theme_name == NULL)
+    theme->metacity_theme_name = g_strdup ("Clearlooks");
 
-  g_free (theme->icon_theme_name);
   theme->icon_theme_name = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
+  if (theme->icon_theme_name == NULL)
+    theme->icon_theme_name = g_strdup ("gnome");
 
-  g_free (theme->cursor_theme_name);
-#ifdef HAVE_XCURSOR
   theme->cursor_theme_name = gconf_client_get_string (client, CURSOR_THEME_KEY, NULL);
+#ifdef HAVE_XCURSOR
   theme->cursor_size = gconf_client_get_int (client, CURSOR_SIZE_KEY, NULL);
-#else
-  theme->cursor_theme_name = gconf_client_get_string (client, CURSOR_FONT_KEY, NULL);
 #endif
+  if (theme->cursor_theme_name == NULL)
+    theme->cursor_theme_name = g_strdup ("default");
 
-  g_free (theme->application_font);
   theme->application_font = gconf_client_get_string (client, APPLICATION_FONT_KEY, NULL);
+
+  return theme;
 }
 
 static gchar *
@@ -352,6 +338,9 @@ theme_is_equal (const GnomeThemeMetaInfo *a, const GnomeThemeMetaInfo *b)
       strcmp (a->cursor_theme_name, b->cursor_theme_name))
     return FALSE;
 
+  if (a->cursor_size != b->cursor_size)
+    return FALSE;
+
   a_set = a->gtk_color_scheme && strcmp (a->gtk_color_scheme, "");
   b_set = b->gtk_color_scheme && strcmp (b->gtk_color_scheme, "");
   if ((a_set != b_set) ||
@@ -381,47 +370,22 @@ theme_set_custom_from_theme (const GnomeThemeMetaInfo *info, AppearanceData *dat
     g_free (custom->gtk_color_scheme);
     g_free (custom->cursor_theme_name);
     g_free (custom->application_font);
-    custom->gtk_theme_name = NULL;
-    custom->icon_theme_name = NULL;
-    custom->metacity_theme_name = NULL;
     custom->gtk_color_scheme = NULL;
-    custom->cursor_theme_name = NULL;
     custom->application_font = NULL;
 
-    if (info->gtk_theme_name)
-      custom->gtk_theme_name = g_strdup (info->gtk_theme_name);
-    else
-      custom->gtk_theme_name = get_default_string_from_key (data->client, GTK_THEME_KEY);
-
-    if (info->icon_theme_name)
-      custom->icon_theme_name = g_strdup (info->icon_theme_name);
-    else
-      custom->icon_theme_name = get_default_string_from_key (data->client, ICON_THEME_KEY);
-
-    if (info->metacity_theme_name)
-      custom->metacity_theme_name = g_strdup (info->metacity_theme_name);
-    else
-      custom->metacity_theme_name = get_default_string_from_key (data->client, METACITY_THEME_KEY);
+    /* these settings are guaranteed to be non-NULL */
+    custom->gtk_theme_name = g_strdup (info->gtk_theme_name);
+    custom->icon_theme_name = g_strdup (info->icon_theme_name);
+    custom->metacity_theme_name = g_strdup (info->metacity_theme_name);
+    custom->cursor_theme_name = g_strdup (info->cursor_theme_name);
+    custom->cursor_size = info->cursor_size;
 
+    /* these can be NULL */
     if (info->gtk_color_scheme)
       custom->gtk_color_scheme = g_strdup (info->gtk_color_scheme);
     else
       custom->gtk_color_scheme = get_default_string_from_key (data->client, COLOR_SCHEME_KEY);
 
-    if (info->cursor_theme_name) {
-      custom->cursor_theme_name = g_strdup (info->cursor_theme_name);
-#ifdef HAVE_XCURSOR
-      custom->cursor_size = info->cursor_size;
-#endif
-    } else {
-#ifdef HAVE_XCURSOR
-      custom->cursor_theme_name = get_default_string_from_key (data->client, CURSOR_THEME_KEY);
-      custom->cursor_size = get_default_int_from_key (data->client, CURSOR_SIZE_KEY, 18);
-#else
-      custom->cursor_theme_name = get_default_string_from_key (data->client, CURSOR_FONT_KEY);
-#endif
-    }
-
     if (info->application_font)
       custom->application_font = g_strdup (info->application_font);
     else
@@ -630,7 +594,7 @@ theme_selection_changed_cb (GtkWidget *icon_view, AppearanceData *data)
   }
 
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "theme_delete"),
-			    gnome_theme_is_writable (theme, GNOME_THEME_TYPE_METATHEME));
+			    theme_is_writable (theme));
   gtk_widget_set_sensitive (glade_xml_get_widget (data->xml, "theme_save"), is_custom);
 }
 
@@ -704,8 +668,7 @@ theme_details_changed_cb (AppearanceData *data)
   gboolean done = FALSE;
 
   /* load new state from gconf */
-  gconf_theme = gnome_theme_meta_info_new ();
-  theme_load_from_gconf (data->client, gconf_theme);
+  gconf_theme = theme_load_from_gconf (data->client);
 
   /* check if it's our currently selected theme */
   icon_view = GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list"));
@@ -868,9 +831,9 @@ themes_init (AppearanceData *data)
   theme_list = gnome_theme_meta_info_find_all ();
   gnome_theme_info_register_theme_change ((ThemeChangedCallback) theme_changed_on_disk_cb, data);
 
+  data->theme_custom = theme_load_from_gconf (data->client);
   data->theme_custom->name = g_strdup (CUSTOM_THEME_NAME);
   data->theme_custom->readable_name = g_strdup_printf ("<i>%s</i>", _("Custom"));
-  theme_load_from_gconf (data->client, data->theme_custom);
 
   for (l = theme_list; l; l = l->next) {
     GnomeThemeMetaInfo *info = l->data;
@@ -960,6 +923,8 @@ themes_init (AppearanceData *data)
   gconf_client_add_dir (data->client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_NONE, NULL);
   gconf_client_add_dir (data->client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_NONE, NULL);
   gconf_client_notify_add (data->client, METACITY_THEME_KEY, (GConfClientNotifyFunc) theme_gconf_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, CURSOR_THEME_KEY, (GConfClientNotifyFunc) theme_gconf_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, CURSOR_SIZE_KEY, (GConfClientNotifyFunc) theme_gconf_changed, data, NULL, NULL);
   gconf_client_notify_add (data->client, BACKGROUND_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
   gconf_client_notify_add (data->client, APPLICATION_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
   gconf_client_notify_add (data->client, DESKTOP_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
